# Python3 Program: requestReport.py

# GPIO 22 is connected to PICAXE input on pin C.3.
# A high pulse on GPIO 22 interrupts the PICAXE processor.
# The PICAXE responds by sending two bytes of raw ADC temperature data.

# The companion PICAXE program is sendReport.bas

import RPi.GPIO as GPIO             #   for the interrupt output (run as root)
import serial
import sys                          #   for KeyboardInterrupt
from time import sleep

IRQ = 22                            #   interrupt output on GPIO 22

GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)              #   use GPIO numbering
GPIO.setup(IRQ,GPIO.OUT)            #   setup the IRQ line as an output

# This function receives two bytes of raw ADC temperature data,
# converts the data to tempC and tempF, and returns the tempF value.
# If a serial timeout has occurred, an invalid data flag (999) is returned.

def getData():

  ser.flushInput()                  #1: empty the serial buffer
  
  GPIO.output(IRQ,GPIO.HIGH)        #   1mS high pulse triggers the interrupt
  sleep(.001)
  GPIO.output(IRQ,GPIO.LOW)

  valid = True                      #   initialize valid data flag

  try:
    hiByte = ord(ser.read())        #2: (non-blocking) fetch hiByte as number
  except:
    valid = False                   #3: if timeout occurs, data is invalid
  try: 
    loByte = ord(ser.read())        #   (non-blocking) fetch loByte as number
  except:
    valid = False                   #   if timeout occurs, data is invalid

  if valid:                         #4: if data is valid
    ADCval = 256 * hiByte + loByte  #   compute 10-bit ADC value
    tempC = .2 * ADCval - 50        #   compute Centigrade temperature
    tempF = 1.8 * tempC + 32        #   compute Fahrenheit temperature
    tempF = round(tempF,1)          #   round temperature to one decimal
    return(tempF)                   #   and return valid value
  
  else:                             #   if data is invalid
    return(999)                     #   return invalid data flag (999)
# Begin Main Program =========================================================

# Set up and open the serial port, with a timeout of 1 second.
ser=serial.Serial('/dev/ttyAMA0',9600,timeout = 1)

while True:
  try:
    tempF = getData()
    if tempF < 999:
      print('The temperature is',str(tempF)+chr(176),'F')
    else:
      print('Warning: The data is invalid!')
      
    # The following loop enables fast response to KeyboardInterrupt 
    for i in range(50):             #   sleep about 5 seconds
      sleep(.1)
    
  except KeyboardInterrupt:         #   ctrl-c terminates program
    print()                         #   print on new line
    print('Program terminated.')
    ser.close()
    GPIO.cleanup()
    break


